/*! \mainpage MLC driver main page
 *
 * \author Tomas Kosan
 * \version 1.0
 * \date 2012-2013
 * \copyright GNU Public License v3, see http://www.gnu.org/licenses/quick-guide-gplv3.html
 *
 * \section intro_sec Introduction
 *
 * This documentation describes developed driver named MLC_drv, which covers all hardware implemented in development platform
 * MLC interface. It allows user of development platform to start it use quickly without knowing low level design of MLC interface.
 * Most of functionality of driver is shown in MLC_test_main.c see this file for reference and example use of driver.
 *
 * \section changes Change log:
 *
 * \subsection changes9 Changes 5.6.2014:
 * - Added address constants for external FPGA PWM module firmware.
 *
 * \subsection changes8 Changes 16.4.2014:
 * - added ADC init procedure to MLC_init() function.
 *
 * \subsection changes7 Changes 21.03.2014:
 *  - added a new structure mlc_adc_result, which is suitable for overlap array of results from ADC. Then it is possible access
 *  each value by real index of input as defined in schematics.
 *
 * \subsection changes6	Changes 13.11.2013:
 * 	- LS3137 now supports full access to CPLD, FPGA and peripherals (ADC, DAC) on MLC interface
 * 	- LS3137 can access external SRAM memory
 * 	- MLC_ functions parameters and return values definitions was rewritten to fixed widths, i.e. unsigned int is now replaced by uint16_t. This
 * 	change solves differences between C2000 and R4F platform integer sizes. Using types with exact number of bits is highly recommended.
 *
 * \subsection changes5 Changes 14.10.2013:
 * 	- added almost full support for TMS320F2812, except DMA transfers (not supported on 2812) and ADC conversion end interrupt.
 * 	- implemented basic support for TMS570LS3137
 * 	- there are now three projects, each for one platform, these depends on each other
 * 	it means that all of them should be imported to workspace !
 *
 * \subsection changes4 Changes 20.8.2013:
 * 	- added functions MLC_DBG_set() and MLC_DBG_clear() see \ref main "Main macros/functions of driver".
 * 	- BTN_xx macros renamed to CHECK_BTNxx()
 * 	- removed flags DMACH1 - DMACH3 user must now implicitly enable each interrupt from DMA by function MLC_DMA_enable_isr()
 * 	- flag USE_DMA changed to USE_ADC_DMA
 *
 * \subsection changes3 Changes 16.8.2013:
 * 	- SCI driver has extended functionality + added support for FIFO, see \ref sci "SCI documentation."
 * 	- MLC_driver was moved to stand-alone CCS project, this behavior allows us to use it as a library
 * 	- also TI sources for 28335 peripherals was moved to project tree of MLC_driver and they are linked to
 * 	  library, therefore you should not use it in project ! See folders includeTI and src, which files are included.
 * 	  However it can be used as part of project, then just copy MLC_drv.c and .h to your project and add also TI sources.
 *
 *
 * \subsection changes2 Changes 1.6.2013:
 * 	- simple SCI driver is now part of MLC_drv, see \ref sci "SCI documentation."
 *
 * \subsection changes1 Changes 21.5.2013:
 * 	- DAC driver is now part of MLC_drv, see \ref dac "DAC documentation."
 * 	- LCD driver is now part of MLC_drv, see \ref lcd "LCD documentation."
 * 	- new debug macros, see \ref debug "DEBUG documentation."
 * 	- full support for DMA transfers and interrupts from AD converters, see \ref adc "ADC documentation."
 * 	- added support for external RAM, it can be size of 128kB (compatible with eZdsp28335) or 256kB, however then button S7 will not work, see \ref ext_ram "documentation."
 * 	- added support for global enable signal, useful for CPLD/FPGA to detect DSP reset condition. Requires CPLD firmware at least 0.4. See \ref main "documentation."
 *
 *
 * \section using_sec Using driver in as part of your project
 *
 * \subsection step1 Step 1: Install driver
 *
 * Driver uses four files, MLC_drv.h, MLC_drv_config.h, platform.h and MLC_drv.c. Add them  to your project, include header file MLC_drv.h and you are done.
 * However driver depends on TI's DSP2833x_Device.h and DSP2833x_Examples.h or DSP281x_Device.h and DSP281x_Examples.h or ARM headers and sources, depending on
 * DSP used. Those have to be added to your project as well. File platform.h is used to define type of MCU used in project. It must be set properly, otherwise
 * project will not compile.
 *
 * \subsection step2 Step 2: Minimal startup code
 *
 * Function MLC_init() must be called first, before any other function of MLC driver. Than we can use rest of functions and macros of driver to configure and
 * access peripherals available on MLC interface board.
 *
 * \section using_sec2 Using driver as library
 *
 * \subsection step12 Step 1: Install driver library - RECOMENDED
 *
 * Driver is standalone CCS static library project, but it is splitted to three supported platforms: MLC_interface_lib, MLC_interface_lib_2812 and MLC_interface_lib_LS3137.
 * This behavior is needed because each platform must have special settings for compiler etc.
 * The MLC_interface_lib project is basic implementation of driver and other two depends on it.
 *
 * This project has to be imported to your workspace and should have name MLC_interface_lib. MLC_interface_lib is suitable for TMS320F28335 development and does not have
 * any additional depencies.
 *
 * Recommended way of using driver libs is:
 * 1/ download sources of demo project and proper library from SVN.
 * - for TMS320F28335: MLC_test_v2 and MLC_interface_lib
 * - for TMS320F2812: MLC_test_v2_2812, MLC_interface_lib_2812 and MLC_interface_lib
 * - for TMS570LS3137: MLC_test_v2_LS3137, MLC_interface_lib_LS3137 and MLC_interface_lib
 *
 * Then import all projects to workspace, let the CCS to copy them to your workspace, otherwise you risk, that project will be changing during time as
 * a new versions will be commited to SVN.
 * All three demos and libraries are setup to work out of the box after sucessfull import to workspace.
 *
 * If somethings goes wrong below is shown setup of demo project for TMS320F28335.
 * Project for TMS320F28335 which wants use this library driver must have these settings:
 * \image html setup_inc.png "1/ Compiler should have include dirs set."
 *
 * \image html setup_lib.png "2/ Linker should have library search set."
 *
 * \image html setup_ref.png "3/ Project should reference to driver library."
 *
 * Simplest way how to use library driver is to create copy of MLC_test_v2 project under different name and replace its source with your sources.
 *
 * \subsection config Configuration of driver
 *
 * Drivers configuration is held in file MLC_drv_config.h. This file consist of defines which controls
 * driver behavior.
 *
 * \subsection quick Quick start
 *
 * Documentation of functions are grouped by purpose in Modules tab, brief documentation can be found \ref MLC "here".
 * You can quickly access documentation for:
 *
 *  - Main macros and functions \ref main "here".
 *  - AD converters \ref adc "here".
 *  - DA converter \ref dac "here".
 *  - LCD display \ref lcd "here".
 *  - External addresses \ref ext_addr "here".
 *  - Debug utils \ref debug "here".
 *  - SCI functions are \ref sci "here"
 *
 *
 * \subsection todo_ TODO:
 *
 * Improve support for TMS570LS3137, add support for RICE BSL system for Hercules MCU.
 */

/*! \file
 * \anchor MLC
 * \brief Header file for MLC driver
 *
 * \details	The MLC driver is used to control hardware platform named MLC (Multi Level Converter) interface.
 * \details It provides all necessary logically named constants to provide easy using of designed hardware.
 * \details The driver also provides functions to configure and use all hardware onboard.
 *
 *
 * \author Tomas Kosan
 * \version 0.9
 * \date 2012-2013
 * \copyright GNU Public License v3, see http://www.gnu.org/licenses/quick-guide-gplv3.html
 *
 *
 *
 */

#ifndef MLC_DRV_H_
#define MLC_DRV_H_

#include "stdint.h"
#include "platform.h"
#include "MLC_drv_config.h"
#include "MLC_ext_module.h"

/** \addtogroup types Definition of driver's types
 * @{
 *
 * \anchor types
 *
 */

/*!
 * Structure which holds internal data of MLC driver.
 */
typedef struct{
	uint16_t cpld_fw_ver; /*!< CPLD firmware version, it is loaded from CPLD by MLC_init() routine. */
	uint16_t fpga_fw_ver; /*!< FPGA firmware version, it is loaded from FPGA by MLC_init() routine. */
	uint16_t pcb_hw_ver;  /*!< PCB version, it is loaded from CPLD by MLC_init() routine. */
	uint16_t adc_conf;    /*!< ADC configuration flags, it is updated each time MLC_ADC_setup() is called. */
	uint16_t mod_fw_ver;  /*!< Firmware version of FPGA PWM module. */
} mlc_info_t;           /*!< Type definition of MLC result structure.*/

/*!
 * Structure which maps array of results to user friendlier structure.
 * It stores 16x16bit wide values, which have 18bit of valid data.
 * High word of data (most significant) is stored in part inX, where X is
 * input number. Last two bits are stored in part inX_low.
 * In most cases inX_low values are ignored by user.
 */
typedef struct{
  int16_t in0;         /*!< ADC input 0 high word of result.*/
  int16_t in0_low;     /*!< ADC input 0 low word of result.*/
  int16_t in1;         /*!< ADC input 1 high word of result.*/
  int16_t in1_low;     /*!< ADC input 1 low word of result.*/
  int16_t in2;         /*!< ADC input 2 high word of result.*/
  int16_t in2_low;     /*!< ADC input 2 low word of result.*/
  int16_t in3;         /*!< ADC input 3 high word of result.*/
  int16_t in3_low;     /*!< ADC input 3 low word of result.*/
  int16_t in4;         /*!< ADC input 4 high word of result.*/
  int16_t in4_low;     /*!< ADC input 4 low word of result.*/
  int16_t in5;         /*!< ADC input 5 high word of result.*/
  int16_t in6_low;     /*!< ADC input 5 low word of result.*/
  int16_t in7;         /*!< ADC input 6 high word of result.*/
  int16_t in7_low;     /*!< ADC input 6 low word of result.*/
  int16_t in8;         /*!< ADC input 7 high word of result.*/
  int16_t in8_low;     /*!< ADC input 7 low word of result.*/
} mlc_adc_result; /*!< Type definition of ADC result structure.*/

/** @}*/


/** \addtogroup globals Definition of global variables
 * @{
 *
 * \anchor glob_vars
 */

extern mlc_info_t* pMLC_info_struct; /*!< Pointer to internal MLC_info structure, which holds informations about MLC interface. */
extern mlc_info_t MLC_info_struct; /*< Internal MLC_info structure, which holds informations about MLC interface. */

/** @}*/

/** \addtogroup addresses Definition of addresses of external peripherals
 * @{
 *
 * \anchor ext_addr
 */
#if MCU_TYPE == MCU_TYPE_TMS320F2812 || MCU_TYPE == MCU_TYPE_TMS320F28335
// ZONE0, address 0x0000
#define BASE_ADDRESS 0x00004000 /*!< Base address of ZONE0, address 0x0000 from outer world*/
#endif

#if MCU_TYPE == MCU_TYPE_TMS570LS3137
#define BASE_ADDRESS 0x60000000u	/*!< Base address of EMIF nCS1, address 0x00000000 from outer world*/
#endif

/*--LATEX-CPLD-BLOCK-START--*/
/*******************************************************************************************/
// CPLD has address offset range 0xF
// write addresses constants
#define WRITE_AD_CONF	0x0000	/*!< Offset of XINTF location of ADC configuration register */
#define WRITE_PWR_OUT	0x0001	/*!< Offset of XINTF location of PWR out output register */
#define WRITE_SETRES_AD	0x0002	/*!< Offset of XINTF location of ADC reset signal to set to active state */
#define WRITE_CLRRES_AD	0x0003	/*!< Offset of XINTF location of ADC reset signal to set to inactive state */
#define WRITE_DBGLEDS	0x0004	/*!< Offset of XINTF location of debug LEDs/pins output register */

// read addresses constants
#define READ_VER		0x0000	/*!< Offset of XINTF location of CPLD firmware version register*/
#define READ_AD1		0x0001	/*!< Offset of XINTF location of AD CH1 read channel*/
#define READ_AD2		0x0002	/*!< Offset of XINTF location of AD CH2 read channel*/
#define READ_AD3		0x0003	/*!< Offset of XINTF location of AD CH2 read channel*/
#define READ_HV_INPUT	0x0004	/*!< Offset of XINTF location of HV input read register*/
#define READ_ARC		0x0005	/*!< Offset of XINTF location of ARC read register*/
#define READ_UIO		0x0009	/*!< Offset of XINTF location of UNI I/O read register*/
#define READ_PCB_REV	0x000F	/*!< Offset of XINTF location of PCB version register*/

// read+write addresses constants
#define RW_CS_EXT1		0x0006	/*!< Offset of XINTF location of EXT1 access*/
#define RW_CS_EXT2		0x0007	/*!< Offset of XINTF location of EXT2 access*/
#define RW_CS_EXT3		0x0008	/*!< Offset of XINTF location of EXT3 access*/
#define RW_UIO			0x0009	/*!< Offset of XINTF location of UIO R/W register*/
#define RW_UIO_CONF		0x000A	/*!< Offset of XINTF location of UIO R/W configuration register */
/*******************************************************************************************/
/*--LATEX-CPLD-BLOCK-END--*/

/*--LATEX-FPGA-BLOCK-START--*/
/*******************************************************************************************/
// FPGA uses addresses higher than 0xF
#define READ_FPGA_VER	0x0010	/*!< Offset of XINTF location of FPGA firmware version read register*/
/*!
 * FPGA LED register offset definition.
 * Register is splitted to two parts: bits 15-8 are configuration, 7-0 are data to show.
 * Configuration data says which data will LEDs show. See FPGA_LEDS... constants.
 * Currently showing of data form lower byte of this register and all PWM gates are supported by
 * FPGA demo firmware 3.2 and higher.
 */
#define RW_FPGA_LED_REG	0x0011

/*!
 * FPGA LCD register offset definition.
 * Register is splitted to two parts: high byte is position where character passed as low byte
 * will be printed.
 */
#define WRITE_LCD_DISP	0x0012	/*!< Offset of XINTF location of LCD write register*/

#define FPGA_RESERVED_13   0x0013 /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_14   0x0014 /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_15   0x0015 /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_16   0x0016 /*!< Reserved for future use of basic FPGA design. */

#define FPGA_FAULTS_ADDR   0x0017 /*!< Address of FAULT register. */

#define FPGA_RESERVED_18   0x0018 /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_19   0x0019 /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_1A   0x001A /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_1B   0x001B /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_1C   0x001C /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_1D   0x001D /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_1E   0x001E /*!< Reserved for future use of basic FPGA design. */
#define FPGA_RESERVED_1F   0x001F /*!< Reserved for future use of basic FPGA design. */


#define FPGA_LEDS_SHOW_DATA		0x0000 /*!< Constant for setting LED entity to show data from its own register*/
#define FPGA_LEDS_SHOW_PWMA		0x0100 /*!< Constant for setting LED entity to show data from PWMA port*/
#define FPGA_LEDS_SHOW_PWMB		0x0200 /*!< Constant for setting LED entity to show data from PWMB port*/
#define FPGA_LEDS_SHOW_PWMC		0x0300 /*!< Constant for setting LED entity to show data from PWMC port*/
#define FPGA_LEDS_SHOW_PWMD		0x0400 /*!< Constant for setting LED entity to show data from PWMD port*/
#define FPGA_LEDS_SHOW_PWME		0x0500 /*!< Constant for setting LED entity to show data from PWME port*/
#define FPGA_LEDS_SHOW_PWMF		0x0600 /*!< Constant for setting LED entity to show data from PWMF port*/

/*******************************************************************************************/
/*--LATEX-FPGA-BLOCK-END--*/

/** @}*/

/** \addtogroup ADC_conf Configuration settings and helper functions for ADC
 * @{
 *
 * \anchor adc
 *
 */

// AD numbering
#define AD_CH1			0x1		/*!< Constant for selecting ADC CH1.*/
#define AD_CH2			0x2		/*!< Constant for selecting ADC CH2.*/
#define AD_CH3			0x4		/*!< Constant for selecting ADC CH3.*/

#define AD_SOCA			0x1		/*!< Constant for selecting ADC CONVST A.*/
#define AD_SOCB			0x2		/*!< Constant for selecting ADC CONVST B.*/
#define AD_SOCC			0x4		/*!< Constant for selecting ADC CONVST C.*/


// AD config values
// oversampling setup values
// no OS
#define AD_OS0			0x0000 	/*!< Constant defining no oversampling for ADC. */
// 2xOS
#define AD_OS1			0x0001	/*!< Constant defining 2x oversampling for ADC. */
// 4xOS
#define AD_OS2			0x0002	/*!< Constant defining 4x oversampling for ADC. */
// 8xOS
#define AD_OS3			0x0003	/*!< Constant defining 8x oversampling for ADC. */
// 16x OS
#define AD_OS4			0x0004	/*!< Constant defining 16x oversampling for ADC. */
// 32xOS
#define AD_OS5			0x0005	/*!< Constant defining 32x oversampling for ADC. */
// 64xOS
#define AD_OS6			0x0006	/*!< Constant defining 64x oversampling for ADC. */
// range select constants
#define AD_RNG_5V		0x0000	/*!< Constant defining +/- 5V input range of ADC. */
#define AD_RNG_10V		0x0001	/*!< Constant defining +/- 10V input range of ADC. */

// Parameters for MLC_setup_AD_SOC() function
// mapping of convstart pins to ADs
// each AD has it own convstart pin
// CH1 - GPIO32/ADSOCA
// CH2 - GPIO33/ADSOCB
// CH3 - GPIO61 - no ADSOC from EV manager

#define AD_SOC_ALL_DEFAULT		0x0000	/*!< CH3 will start with toggling of ADSOC pin - no HW SOC support */
#define AD_SOC_CH3_SOCA			0x1000	/*!< CH1 + CH3 share ADSOCA, CH2 uses default ADSOCB */
#define AD_SOC_CH3_SOCB			0x2000	/*!< CH1 uses ADSOCA, CH2 + CH3 share default ADSOCB */
#define AD_SOC_CH3_SOCAB		0x3000	/*!< CH3 will start with CH1 or CH2 and uses ADSOCA or ADSOCB in the same time */
#define AD_SOC_ALL_SOCA			0x4000	/*!< CH1 + CH2 + CH3 will start with ADSOCA */
#define AD_SOC_ALL_SOCB			0x5000	/*!< CH1 + CH2 + CH3 will start with ADSOCB */

// This flag turns on use of EOC interrupt
// It will use XINT1
//#define USE_AD_INTERRUPT	1	/*!< Allow ADC end of conversion interrupt */

#define AD_IN0		0			/*!< Index of MSB data from ADC input 0 */
#define AD_IN1		2			/*!< Index of MSB data from ADC input 1 */
#define AD_IN2		4			/*!< Index of MSB data from ADC input 2 */
#define AD_IN3		6			/*!< Index of MSB data from ADC input 3 */
#define AD_IN4		8			/*!< Index of MSB data from ADC input 4 */
#define AD_IN5		10			/*!< Index of MSB data from ADC input 5 */
#define AD_IN6		12			/*!< Index of MSB data from ADC input 6 */
#define AD_IN7		14			/*!< Index of MSB data from ADC input 7 */

#define DMA_ADCH1	AD_CH1			/*!< Constant defining DMA CH1 */
#define DMA_ADCH2	AD_CH2			/*!< Constant defining DMA CH2 */
#define DMA_ADCH3	AD_CH3			/*!< Constant defining DMA CH3 */


/** @}*/

/** \addtogroup CONST Constants of driver
 * @{
 */
#define PCB_REV_1		0x1		/*!< Constant with value of PCB revision. This constant is used to enable/disable GPIO function depending on revision of PCB.*/
#define PCB_REV_2		0x2		/*!< Constant with value of PCB revision. This constant is used to enable/disable GPIO function depending on revision of PCB.*/
#define PCB_REV_3		0x3		/*!< Constant with value of PCB revision. This constant is used to enable/disable GPIO function depending on revision of PCB.*/


/** @}*/

/** \addtogroup HELPER Helper macros/functions of driver
 * @{
 */

// define buttons, they are active at 0, data is inverted
// i.e. pushed button means that BTN_Sx returns '1'
#if MCU_TYPE == MCU_TYPE_TMS320F28335
	#define CHECK_BTN_S5()		!GpioDataRegs.GPADAT.bit.GPIO28	/*!< This macro returns 1 when button S5 is pressed.*/
	#define CHECK_BTN_S8()		!GpioDataRegs.GPADAT.bit.GPIO29	/*!< This macro returns 1 when button S8 is pressed.*/
	#define CHECK_BTN_S6()		!GpioDataRegs.GPADAT.bit.GPIO30	/*!< This macro returns 1 when button S6 is pressed.*/

	#if USE_256kB_RAM == 1
		#define	CHECK_BTN_S7()		0								/*!< If we want use ext. RAM > 128kB, then it always returns 0.*/
	#else
		#define CHECK_BTN_S7()		!GpioDataRegs.GPADAT.bit.GPIO31 /*!< This macro returns 1 when button S7 is pressed. Collide with ext. RAM > 128kB.*/
	#endif
#endif


#if MCU_TYPE == MCU_TYPE_TMS320F2812
#define CHECK_BTN_S5()		!GpioDataRegs.GPFDAT.bit.GPIOF8	 /*!< This macro returns 1 when button S5 is pressed.*/
#define CHECK_BTN_S8()		!GpioDataRegs.GPFDAT.bit.GPIOF9	 /*!< This macro returns 1 when button S8 is pressed.*/
#define CHECK_BTN_S6()		!GpioDataRegs.GPFDAT.bit.GPIOF10 /*!< This macro returns 1 when button S6 is pressed.*/
#define CHECK_BTN_S7()		!GpioDataRegs.GPFDAT.bit.GPIOF11 /*!< This macro returns 1 when button S7 is pressed. Collide with ext. RAM > 128kB.*/
#endif

#if MCU_TYPE == MCU_TYPE_TMS570LS3137
#define CHECK_BTN_S5()		gioGetBit(gioPORTA, 3)	/*!< This macro returns 1 when button S5 is pressed.*/
#define CHECK_BTN_S8()		gioGetBit(gioPORTA, 2)	/*!< This macro returns 1 when button S8 is pressed.*/
#define CHECK_BTN_S6()		gioGetBit(gioPORTA, 4)	/*!< This macro returns 1 when button S6 is pressed.*/
#define CHECK_BTN_S7()		gioGetBit(gioPORTA, 0)  /*!< This macro returns 1 when button S7 is pressed.*/

void DELAY_US(float us); /*!< This function provides simple delay up to 330s with us resolution, however it is not much precise if delay < 10us. */

#endif


/** @}*/


/** \addtogroup MAIN Main macros/functions of driver
 * @{
 * \anchor main
 */

/*!
 *
 * This function should be called first, it initializes XINTF ZONE0, ZONE7 (if ext. RAM is used) and fixed purpose GPIOs.
 */
void MLC_init();

void MLC_global_enable(); 	/*!< Use GPIO60 to enable CPLD and FPGA functionality. Useful for detecting DSP reset via JTAG. Requires CPLD firmware at least 0.4 */
void MLC_global_disable();	/*!< Use GPIO60 to disable CPLD and FPGA functionality. Useful for detecting DSP reset via JTAG. Requires CPLD firmware at least 0.4 */

/*!
 * Turn on PWR outputs according to ones in pwr parameter.
 * @param pwr defines which outputs should be turn on, in fact this means, that output will be zero level. But those power outputs are open collector
 * so turn on means connect to ground.
 */
void MLC_PWR_on(uint16_t pwr);

/*!
 * Turn off PWR outputs according to ones in pwr parameter.
 * @param pwr defines which outputs should be turn off, in fact this means, that output will be floating, because those power outputs are open collector
 * so turn off means disconnect from ground.
 */
void MLC_PWR_off(uint16_t pwr);

/*!
 * Set debug outputs according to ones in dbg parameter.
 * @param dbg defines which debug outputs should be set to 1. I.e. passing 0x01 will turn on debug LED V1.
 */
void MLC_DBG_set(uint16_t dbg);

/*!
 * Clear debug outputs according to ones in dbg parameter.
 * @param dbg defines which debug outputs should be set to 0. I.e. passing 0x01 will turn off debug LED V1.
 */
void MLC_DBG_clear(uint16_t dbg);

// read/write macros - subtitute of inline functions, which do not work properly
// these macros allows to comunicate at rate about 20MB/s
/*!
 * Macro for writing to XINTF/EMIF connected peripherals. CAUTION ! Macro casts it's data parameter to UNSIGNED INT of 16b wide.
 * If design requires writing of SIGNED INT, then data can be UNSIGNED INT without problem, however automatic cast from other
 * data types (i.e. float) will not work. They will be always positive.
 * @param addr uint16_t address of external peripheral, use defined constants WRITE_*
 * @param data uint16_t data to write to peripheral
 */
#define MLC_WRITE(addr, data) *((volatile uint16_t*)BASE_ADDRESS+addr)=(uint16_t)(data)
/*!
 * Macro for reading from XINTF/EMIF connected peripherals.
 * @param addr address of external peripheral, use defined constants READ_*
 * @return read from peripheral
 */
#define MLC_READ(addr) *((volatile uint16_t*)BASE_ADDRESS+addr)

// ordinary functions with same behaviour as macros above
// there are slower (~4MB/s) but, saves some memory
/*!
 * Function for writing to XINTF connected peripherals. However using macro MLC_WRITE() is recommended due higher performance.
 * @param addr address of external peripheral, use defined constants WRITE_*
 * @param data data to write to periheral
 */
__inline void MLC_write(uint16_t addr, uint16_t data) {*((volatile uint16_t*)(BASE_ADDRESS+addr))=(uint16_t)data;}
/*!
 * Function for reading from XINTF connected peripherals. However using macro MLC_READ() is recommended due higher performance.
 * @param addr address of external peripheral, use defined constants READ_*
 * @return value from periheral
 */
__inline uint16_t MLC_read(uint16_t addr) {return *((volatile uint16_t*)(BASE_ADDRESS+addr));}

/*!
 * Macro for reading FAULT inputs from converters.
 */
#define FPGA_check_faults() MLC_READ(FPGA_FAULTS_ADDR)

/*!
 * Constant defining value returned by FPGA_check_faults() when no FAULT is active.
 */
#define NO_FAULTS 0x3F

/** @}*/

/** \addtogroup ADC_conf Configuration settings and helper functions for ADC
 * @{
 * \anchor adc
 */

/*!
 * This function encapsulate reset process of ADCs.
 */
void MLC_ADC_reset(void);

/*!
 * Function for setting oversampling and input range of all three ADCs.
 *
 * @param os_1, os_2, os_3 oversampling setup for each channel, use AD_OS0 - AD_OS6. Oversampling will prolong conversion time. I.e. if AD_OS6
 * is set, then conversion time will be 2^6*4us = 256us ! All ADC shares one BUSY signal, therefore conversion end is detected after slowest
 * conversion finished.
 * @param rng_1, rng_2, rng_3 set input range, use AD_RNG_5V or AD_RNG_10V
 */
void MLC_ADC_setup(unsigned int os_1, unsigned int os_2, unsigned int os_3, unsigned int rng_1, unsigned int rng_2, unsigned int rng_3);

/*!
 * Function for setting CONVST source pin. Parameter defines which HW pin starts which ADC channel, valid combinations
 * are defined by AD_SOC_* constants.
 * @param adsoc is one of AD_SOC_* constants.
 */
void MLC_ADC_setup_SOC(uint16_t adsoc);

/*!
 * Function for manually start conversion of all three channels. In this case settings setted by calling MLC_ADC_setup_SOC() has no meaning.
 * All three channels will be always started.
 */
void MLC_ADC_start_conv(void);

/*!
 * Function for manually start conversion of channel. Which ADC(s) will start measure is defined by MLC_ADC_setup_SOC() function.
 * @param channel is one of AD_CHx constants.
 */
void MLC_ADC_start_one_conv(uint16_t channel);

/*! Reads measured values from ADC No.1
 * this function triggers full read sequence and update values
 * of array of AD results
 * @return address of array with results
 */
volatile int16_t* MLC_ADC1_read(void);

/*! Reads measured values from ADC No.2
 * this function triggers full read sequence and update values
 * of array of AD results
 * @return address of array with results
 */
volatile int16_t* MLC_ADC2_read(void);

/*! Reads measured values from ADC No.3
 * this function triggers full read sequence and update values
 * of array of AD results
 * @return address of array with results
 */
volatile int16_t* MLC_ADC3_read(void);

/*! Returns pointer to array where measured values are stored,
 * this function will NOT actually read a new data from AD1 !
 * @return address of array with results
 */
volatile int16_t* MLC_ADC1_get_res_ptr(void);

/*! Returns pointer to array where measured values are stored,
 * this function will NOT actually read a new data from AD2 !
 * @return address of array with results
 */
volatile int16_t* MLC_ADC2_get_res_ptr(void);

/*! Returns pointer to array where measured values are stored,
 * this function will NOT actually read a new data from AD3 !
 * @return address of array with results
 */
volatile int16_t* MLC_ADC3_get_res_ptr(void);

/*! Returns pointer to structure where measured values are stored,
 * this function will NOT actually read a new data from AD1 !
 * @return address of structure mlc_adc_result with results
 */
volatile mlc_adc_result* MLC_ADC1_get_res_strptr(void);

/*! Returns pointer to structure where measured values are stored,
 * this function will NOT actually read a new data from AD2 !
 * @return address of structure mlc_adc_result with results
 */
volatile mlc_adc_result* MLC_ADC2_get_res_strptr(void);

/*! Returns pointer to structure where measured values are stored,
 * this function will NOT actually read a new data from AD3 !
 * @return address of structure mlc_adc_result with results
 */
volatile mlc_adc_result* MLC_ADC3_get_res_strptr(void);

/*! This function can be used to save some computing time.
 * Use as MLC_read_AD(AD_res_1,READ_AD1);
 * @param ad_res pointer to array where to store results. The array must have size of 16 words
 * @param ad_addr address of ADC in memory, use READ_ADx constant.
 * @return pointer to array passed as ad_res
 */
volatile int16_t* MLC_ADC_read(volatile int16_t* ad_res, uint16_t ad_addr);

/*! Waits for all AD converters finished conversion and while waiting it blocks.
 * @return number of while cycles spent in blocked state.
 */
unsigned int MLC_ADC_wait(void);

// helper function, it calculate voltage from measured value
/*!
 *  Helper function, it calculate voltage from measured value.
 *  @param value measured value from ADC, 16 bits
 *  @param range use AD_RNG_5V or AD_RNG_10V
 *  @return real value measured by ADC
 */
float MLC_ADC_calc_volt(int16_t value, uint16_t range);

/*!
 * Function for setup of handler function called after conversion of AD ends. The handler has to acknowledge interrupt by command PieCtrlRegs.PIEACK.all = PIEACK_GROUP1;
 *
 * TMS320F28335 - it uses XINT1 - i.e. int1.4. This interrupt should not be used in user code.
 * @param handler pointer to function to call when conversion end. It MUST be defined with flag interrupt.
 */
void MLC_ADC_enable_isr(interrupt void* handler);

/*!
 * Function for disabling AD end of conversion interrupt.
 */
void MLC_ADC_disable_isr(void);


// F2812 lacks DMA support
#if MCU_TYPE == MCU_TYPE_TMS320F28335  || MCU_TYPE == MCU_TYPE_TMS570LS3137
/*!
 * Enables DMA interrupt and specify user handler. Interrupt handler must acknowledge interrupt at least by command PieCtrlRegs.PIEACK.all = PIEACK_GROUP7;
 *
 * If DMA transfer is enabled, then it is automatically started after ADC conversion ends.
 * @param channel DMA channel to enable, use one of constant of DMA_ADCHx
 * @param handler pointer to function to call when data transfer end. It MUST be defined with flag interrupt.
 */
void MLC_DMA_enable_isr(uint16_t channel, interrupt void* handler);

/*!
 * Disables DMA interrupt for specified channel.
 * @param channel DMA channel to disable, use one of constant of DMA_ADCHx
 */
void MLC_DMA_disable_isr(uint16_t channel);

/*!
 * Function for checking DMA transfer activity.
 * @return if > 0 then DMA transfer is active
 */
uint16_t MLC_DMA_active(void);

/*!
 * Activate start of DMA transfer after ADC conversion end.
 */

void MLC_DMA_activate(void);

/*!
 * Disable start of DMA transfer after ADC conversion end.
 */

void MLC_DMA_deactivate(void);


/** @}*/

#endif

/** \addtogroup LCD_utils Utils for LCD display text output.
 * @{
 * \anchor lcd
 */

// functions for LCD display, FPGA must have basic design to those functions work
/*!
 * Write one char to exact position of LCD display.
 * @param pos position in range <0,31> to write to, display entity uses linear addressing.
 * @param chr character which will be written to position
 */
void MLC_LCD_write_to(uint16_t pos, char chr);
/*!
 * Write string from exact position of LCD display, display entity uses linear addressing.
 * @param pos position in range <0,31> where to start
 * @param pStr string to be written
 */
void MLC_LCD_write_str(char* pStr, uint16_t pos);

/*!
 * Clear display.
 */
void MLC_LCD_clrscr(void);

/** @}*/

/** @}*/

/** \addtogroup DAC_utils Utils for DAC real time output.
 * @{
 * \anchor dac
 */

// constants corresponding
// with PCB marking of ADC outputs
#define	DAC_CH_A	0	/*!< Constant for access channel A of DAC. */
#define DAC_CH_B	1	/*!< Constant for access channel B of DAC. */
#define DAC_CH_C	2	/*!< Constant for access channel C of DAC. */
#define DAC_CH_D	3	/*!< Constant for access channel D of DAC. */
#define DAC_CH_E	4	/*!< Constant for access channel E of DAC. */
#define DAC_CH_F	5	/*!< Constant for access channel F of DAC. */
#define DAC_CH_G	6	/*!< Constant for access channel G of DAC. */
#define DAC_CH_H	7	/*!< Constant for access channel H of DAC. */

extern uint16_t dac_values[8];	/*!< Array which stores values to send to channel defined in dac_sel_ch */
extern uint16_t dac_sel_ch[8];	/*!< Array which maps values from dac_values array to DAC outputs */

void MLC_DAC_reset(void);				/*!< Reset DAC, this must be called before any access to DAC. */
/*!
 *  Sends control word to DAC, according to datasheet of AD5328.
 *  @param cmd word to send as configuration.
 */
void MLC_DAC_control(uint16_t cmd);
/*!
 * Function to send data from buffer to DAC. It can be defined how many values we need to write out.
 * @param count number in range <0,7>. It then send values from dac_values to channel defined in dac_sel_ch from 0 to count.
 * I.e. count = 4 sends values from 0 to 4.
 */
void MLC_DAC_write(uint16_t count);
/*!
 * Initializes DAC, setup it according to flags DAC_USE_x. It also set array of
 * channels to default state, where value with index 0 will be send to DAC_CH_A etc.
 * This means, that we do not need to write dac_sel_ch array.
 */
void MLC_DAC_init(void);

#define dac_reset() 				MLC_DAC_reset()				/*!< Backward compatibility macro */
#define dac_control					MLC_DAC_control 			/*!< Backward compatibility macro */
#define dac_write 					MLC_DAC_write				/*!< Backward compatibility macro */
#define write_dac 					MLC_DAC_write				/*!< Backward compatibility macro */
#define dac_init()					MLC_DAC_init()				/*!< Backward compatibility macro */
#define dac_sel_channel				dac_sel_ch					/*!< Backward compatibility macro */
#define dac_variables				dac_values					/*!< Backward compatibility macro */

/** @}*/


/** \addtogroup DEBUG_utils Utils for simple debug messaging to CIO of CCS.
 * @{
 * \anchor debug
 *
 * They will output string passed as parameter to CCSs console.
 * These macros depends on <stdio.h> and they have a BIG overhead together with big memory footprint ! They also work only in debug mode and uses
 * one breakpoint. Use at your own risk.
 */

#define DEBUG_LEVEL_NONE	0 /*!< Constant defining that debug output will not print any messages. */
#define DEBUG_LEVEL_ERR		1 /*!< Constant defining that debug output will print error messages only. */
#define DEBUG_LEVEL_WRN		2 /*!< Constant defining that debug output will print error and warning messages. */
#define DEBUG_LEVEL_INFO	3 /*!< Constant defining that debug output will print all messages. */

#if DEBUG_LEVEL == DEBUG_LEVEL_INFO
	#define DBG_ERR_PUTS(str)	puts(str)	/*!< Send error str to CCS CIO. Requires <stdio.h> to be included. */
	#define DBG_WRN_PUTS(str)	puts(str)	/*!< Send warning str to CCS CIO. Requires <stdio.h> to be included. */
	#define DBG_INFO_PUTS(str)	puts(str)	/*!< Send info str to CCS CIO. Requires <stdio.h> to be included. */
#endif

#if DEBUG_LEVEL == DEBUG_LEVEL_WRN
	#define DBG_ERR_PUTS(str)	puts(str)	/*!< Send error str to CCS CIO. Requires <stdio.h> to be included. */
	#define DBG_WRN_PUTS(str)	puts(str)	/*!< Send warning str to CCS CIO. Requires <stdio.h> to be included. */
	#define DBG_INFO_PUTS(str)	;			/*!< Send info str to CCS CIO. Requires <stdio.h> to be included. */
#endif

#if DEBUG_LEVEL == DEBUG_LEVEL_ERR
	#define DBG_ERR_PUTS(str)	puts(str)	/*!< Send error str to CCS CIO. Requires <stdio.h> to be included. */
	#define DBG_WRN_PUTS(str)	;			/*!< Send warning str to CCS CIO. Requires <stdio.h> to be included. */
	#define DBG_INFO_PUTS(str)	;			/*!< Send info str to CCS CIO. Requires <stdio.h> to be included. */
#endif

#if DEBUG_LEVEL == DEBUG_LEVEL_NONE
	#define DBG_ERR_PUTS(str)	;			/*!< Send error str to CCS CIO. Requires <stdio.h> to be included. */
	#define DBG_WRN_PUTS(str)	;			/*!< Send warning str to CCS CIO. Requires <stdio.h> to be included. */
	#define DBG_INFO_PUTS(str)	;			/*!< Send info str to CCS CIO. Requires <stdio.h> to be included. */
#endif

/** @}*/

/** \addtogroup SCI_utils	Utils for USB to SCI
 * @{
 * \anchor sci
 */

/*!
 * It initializes GPIO and baudrate generator.
 * @param baudrate required baudrate, for example 115200
 */
void MLC_SCI_init(uint32_t baudrate);
/*!
 * Send one char/byte via SCI -> USB -> PC.
 * This is blocking function, if TX buffer is not empty, then it waits for TX buffer to be empty. If FIFO enabled, then
 * it can feed upto 16 bytes to FIFO before it blocks another write to FIFO and waits until at least one level is empty.
 * @param c	char to send
 */
void MLC_SCI_send_char(char c);
/*!
 * Sends string via SCI -> USB- > PC.
 * This function is blocking and returns after data is sent.
 * @param str pointer to string to send
 */
void MLC_SCI_send_str(char* str);
/*!
 * Waits for one char to be received. Blocks until char is received.
 * @return received data byte/char
 */
char MLC_SCI_recv_char(void);

/*!
 * Check if char is available in RX buffer of SCI.
 * @return 0 if not, >0 if char is in buffer and can be read. If FIFO is enabled it actually returns number of waiting bytes.
 */
char MLC_SCI_char_avail(void);

/*!
 * Waits for char to be send.
 * @return 1 if char was sent.
 */
char MLC_SCI_wait_send(void);

/*!
 * Send one char and waits till it is sent.
 * @param c	char to send
 */
void MLC_SCI_send_char_wait(char c);

/** @}*/

#endif /* end of MLC_drv */

