/*
 * MLC_drv_config.h
 *
 *  Created on: 19.8.2013
 *      Author: Tomas Kosan
 */

/*! \file
 * \anchor MLC
 * \brief Header configuration file for MLC driver
 *
 * \details	The MLC driver is used to control hardware platform named MLC (Multi Level Converter) interface.
 * \details It provides all necessary logically named constants to provide easy using of designed hardware.
 * \details The driver also provides functions to configure and use all hardware onboard.
 * \details This file carries all configuration switches of MLC_drv. It means that this file is intended to
 * \details be used by user to fit MLC_drv to specific project.
 *
 * \author Tomas Kosan
 * \version 0.1
 * \date 2012-2013
 * \copyright GNU Public License v3, see http://www.gnu.org/licenses/quick-guide-gplv3.html
 *
 *
 *
 */

#ifndef MLC_DRV_CONFIG_H_
#define MLC_DRV_CONFIG_H_

/** \addtogroup CONF Configuration settings of driver
 * @{
 */

/*!
 * Allow DMA for ADC conversion results transfer. This flag ensures proper configuration of DMA transfers. ADC uses
 * channels CH1 - CH3.
 * Interrupt after DMA transfer end has to be individualy enabled for each DMA transfer channel by function MLC_enable_DMA_isr().
 */
#define USE_ADC_DMA			1

/*!
 *  Set to 1 to check first data of ADC, usefull when we do not read all data from A/D but it can be tricky, it is better always read all data.
 *  This flag has no meaning when DMA transfers are used.
 */
#define USE_FRSTDATA		0

/*!
 * Set to 1 to enable quick read, reading of 16 values takes ~1,7us. Set to 0 to save some memory, but prolong reading to ~2,5us.
 * This flag has no meaning when DMA transfers are used.
 */
#define QUICK_READ			1

/*!
 *  Set to 1 to allow lower XINTF timings, i.e. quicker reads and writes. Allow only when CPLD has a new version of FW, at least version 4.
 *  MLC_init() however checks version of CPLD firmware and automatically enables this speedup when version is correct. If there
 *  is some issues with reading data from ADC (usually swapped low and high word of result), then disabling this feature is recommended.
 */
#define ALLOW_XINTF_SPEEDUP 0

/*!
 *  If > 0 then PCB revision is forced to be value of FORCE_REV. Useful only for debug.
 */
#define FORCE_PCB_REV		0x0

/*!
 *  Allow using of external RAM, set to 1 to configure XINTF to work with ext. RAM.
 */
#define USE_EXT_RAM 		1

/*!
 *  If set to 1 then it disable BTN_S7, but extend ext. memory size to 256kB,
 *  this flag is applicable to eZdsp 28335 kit replacement by T.Kosan only.
 */
#define USE_256kB_RAM		1

/*!
 *  If set to 1 then Vcc = 5V is applied as ref. signal for DAC. Allowing this flag automatically disables DAC_USE_GAINx2 option.
 */
#define DAC_USE_VCC_AS_REF	0

/*!
 *  Turn on(1)/off(0) internal reference voltage buffers of DAC.
 */
#define DAC_USE_REF_BUFFER	1

/*!
 *  Set output stage gain of DAC to 2x when 1, else 1x. This is useful when Vref is 2,5V and we
 *  demand output swing of DAC in range 0 to 5V.
 *  When option DAC_USE_VCC_AS_REF is set to 1,
 *  then this option has no meaning and gain will be always 1x.
 */
#define DAC_USE_GAINx2		1

/*!
 * This option allows using of TX FIFO of SCI.
 * FIFO is 16 bytes deep and can significantly decrease CPU overhead.
 */
#define USE_SCI_TX_FIFO 	1

/*!
 *  This option allows using of RX FIFO of SCI.
 *  FIFO is 16 bytes deep and can help avoid loosing of data received.
 */
#define USE_SCI_RX_FIFO 	1

/** @}*/


/** \addtogroup DEBUG_utils Utils for simple debug messaging to CIO of CCS.
 * @{
 * \anchor debug
 *
 * They will output string passed as parameter to CCSs console.
 * These macros depends on <stdio.h> and they have a BIG overhead together
 * with big memory footprint ! They also work only in debug mode and uses
 * one breakpoint. Use at your own risk.
 */

#define DEBUG_LEVEL			DEBUG_LEVEL_ERR /*!< Sets desired debug level. */

/** @}*/

#endif /* MLC_DRV_CONFIG_H_ */
